@@ -1,7 +1,7 @@
 /*
  * wee-debug.c - debug functions
  *
- * Copyright (C) 2003-2013 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -29,6 +29,13 @@
 #endif
 #include <string.h>
 #include <time.h>
+#include <gcrypt.h>
+#include <curl/curl.h>
+#include <zlib.h>
+
+#ifdef HAVE_GNUTLS
+#include <gnutls/gnutls.h>
+#endif
 
 #include "weechat.h"
 #include "wee-backtrace.h"
@@ -77,15 +84,9 @@ debug_dump (int crash)
 
     log_printf ("");
     if (crash)
-    {
-        log_printf ("******             WeeChat CRASH DUMP              ******");
-        log_printf ("****** Please send this file to WeeChat developers ******");
-        log_printf ("******    and explain when this crash happened     ******");
-    }
+        log_printf ("****** WeeChat CRASH DUMP ******");
     else
-    {
-        log_printf ("******            WeeChat dump request             ******");
-    }
+        log_printf ("****** WeeChat dump request ******");
 
     gui_window_print_log ();
     gui_buffer_print_log ();
@@ -109,7 +110,7 @@ debug_dump (int crash)
     plugin_print_log ();
 
     log_printf ("");
-    log_printf ("******             End of WeeChat dump             ******");
+    log_printf ("****** End of WeeChat dump ******");
     log_printf ("");
 }
 
@@ -156,12 +157,12 @@ debug_sigsegv ()
                               weechat_home);
     string_iconv_fprintf (stderr, "***\n");
     string_iconv_fprintf (stderr, "*** Please help WeeChat developers to fix this bug:\n");
-    string_iconv_fprintf (stderr, "***   1. If you have a core file, please run:  gdb weechat-curses core\n");
-    string_iconv_fprintf (stderr, "***      then issue \"bt\" command and send result to developers\n");
-    string_iconv_fprintf (stderr, "***      To enable core files with bash shell: ulimit -c 10000\n");
-    string_iconv_fprintf (stderr, "***   2. Otherwise send backtrace (below) and weechat.log\n");
-    string_iconv_fprintf (stderr, "***      (be careful, private info may be in this file since\n");
-    string_iconv_fprintf (stderr, "***      part of chats are displayed, so remove lines if needed)\n\n");
+    string_iconv_fprintf (stderr, "***   1. If you have a core file, please run:  gdb /path/to/weechat core\n");
+    string_iconv_fprintf (stderr, "***      then issue command: \"bt full\" and send result to developers\n");
+    string_iconv_fprintf (stderr, "***      (see user's guide for more info about report of crashes).\n");
+    string_iconv_fprintf (stderr, "***   2. Otherwise send backtrace (below), only if it is a complete trace.\n");
+    string_iconv_fprintf (stderr, "***      Keep the crash log file, just in case developers ask you some info\n");
+    string_iconv_fprintf (stderr, "***      (be careful, private info like passwords may be in this file).\n\n");
 
     weechat_backtrace ();
 
@@ -169,26 +170,6 @@ debug_sigsegv ()
     weechat_shutdown (EXIT_FAILURE, 1);
 }
 
-/*
- * Callback for signal "debug_buffer".
- *
- * This function is called when command "/debug buffer" is issued.
- */
-
-int
-debug_buffer_cb (void *data, const char *signal, const char *type_data,
-                 void *signal_data)
-{
-    /* make C compiler happy */
-    (void) data;
-    (void) signal;
-    (void) type_data;
-
-    gui_buffer_dump_hexa ((struct t_gui_buffer *)signal_data);
-
-    return WEECHAT_RC_OK;
-}
-
 /*
  * Displays tree of windows (this function must not be called directly).
  */
@@ -253,27 +234,6 @@ debug_windows_tree ()
     debug_windows_tree_display (gui_windows_tree, 1);
 }
 
-/*
- * Callback for signal "debug_windows".
- *
- * This function is called when command "/debug windows" is issued.
- */
-
-int
-debug_windows_cb (void *data, const char *signal, const char *type_data,
-                  void *signal_data)
-{
-    /* make C compiler happy */
-    (void) data;
-    (void) signal;
-    (void) type_data;
-    (void) signal_data;
-
-    debug_windows_tree ();
-
-    return WEECHAT_RC_OK;
-}
-
 /*
  * Displays information about dynamic memory allocation.
  */
@@ -369,10 +329,8 @@ debug_hdata_map_cb (void *data, struct t_hashtable *hashtable,
     gui_chat_printf (NULL,
                      "  hdata 0x%lx: \"%s\", %d vars, %d lists:",
                      ptr_hdata, (const char *)key,
-                     hashtable_get_integer (ptr_hdata->hash_var,
-                                            "items_count"),
-                     hashtable_get_integer (ptr_hdata->hash_list,
-                                            "items_count"));
+                     ptr_hdata->hash_var->items_count,
+                     ptr_hdata->hash_list->items_count);
 
     /* display lists */
     hashtable_map (ptr_hdata->hash_list,
@@ -412,7 +370,7 @@ debug_hdata ()
 {
     int count;
 
-    count = hashtable_get_integer (weechat_hdata, "items_count");
+    count = weechat_hdata->items_count;
 
     gui_chat_printf (NULL, "");
     gui_chat_printf (NULL, "%d hdata in memory", count);
@@ -539,6 +497,70 @@ debug_infolists ()
     }
 }
 
+/*
+ * Callback for signal "debug_libs": displays infos about external libraries
+ * used (called when command "/debug libs" is issued).
+ *
+ * Note: this function displays libraries for WeeChat core only: plugins can
+ * catch this signal to display their external libraries.
+ */
+
+int
+debug_libs_cb (void *data, const char *signal, const char *type_data,
+               void *signal_data)
+{
+    /* make C compiler happy */
+    (void) data;
+    (void) signal;
+    (void) type_data;
+    (void) signal_data;
+
+    gui_chat_printf (NULL, "  core:");
+
+    /* display ncurses version */
+    gui_main_debug_libs ();
+
+    /* display gcrypt version */
+#ifdef GCRYPT_VERSION
+    gui_chat_printf (NULL, "    gcrypt: %s%s",
+                     GCRYPT_VERSION,
+                     (weechat_no_gcrypt) ? " (not initialized)" : "");
+#else
+    gui_chat_printf (NULL, "    gcrypt: (?)%s",
+                     (weechat_no_gcrypt) ? " (not initialized)" : "");
+#endif
+
+    /* display gnutls version */
+#ifdef HAVE_GNUTLS
+#ifdef GNUTLS_VERSION
+    gui_chat_printf (NULL, "    gnutls: %s%s",
+                     GNUTLS_VERSION,
+                     (weechat_no_gnutls) ? " (not initialized)" : "");
+#else
+    gui_chat_printf (NULL, "    gnutls: (?)%s",
+                     (weechat_no_gnutls) ? " (not initialized)" : "");
+#endif
+#else
+    gui_chat_printf (NULL, "    gnutls: (not available)");
+#endif
+
+    /* display curl version */
+#ifdef LIBCURL_VERSION
+    gui_chat_printf (NULL, "    curl: %s", LIBCURL_VERSION);
+#else
+    gui_chat_printf (NULL, "    curl: (?)");
+#endif
+
+    /* display zlib version */
+#ifdef ZLIB_VERSION
+    gui_chat_printf (NULL, "    zlib: %s", ZLIB_VERSION);
+#else
+    gui_chat_printf (NULL, "    zlib: (?)");
+#endif
+
+    return WEECHAT_RC_OK;
+}
+
 /*
  * Displays WeeChat directories.
  */
@@ -562,7 +584,11 @@ debug_directories ()
 void
 debug_init ()
 {
-    hook_signal (NULL, "debug_dump", &debug_dump_cb, NULL);
-    hook_signal (NULL, "debug_buffer", &debug_buffer_cb, NULL);
-    hook_signal (NULL, "debug_windows", &debug_windows_cb, NULL);
+    /*
+     * hook signals with high priority, to be sure they will be used before
+     * plugins (they should anyway because this function is called before load
+     * of plugins)
+     */
+    hook_signal (NULL, "2000|debug_dump", &debug_dump_cb, NULL);
+    hook_signal (NULL, "2000|debug_libs", &debug_libs_cb, NULL);
 }
