@@ -1002,15 +1002,19 @@ weechat_aspell_debug_libs_cb (void *data, const char *signal,
     (void) signal_data;
 
 #ifdef USE_ENCHANT
+#ifdef HAVE_ENCHANT_GET_VERSION
     weechat_printf (NULL, "  %s: enchant %s",
                     ASPELL_PLUGIN_NAME, enchant_get_version ());
+#else
+    weechat_printf (NULL, "  %s: enchant (?)", ASPELL_PLUGIN_NAME);
+#endif /* HAVE_ENCHANT_GET_VERSION */
 #else
 #ifdef HAVE_ASPELL_VERSION_STRING
     weechat_printf (NULL, "  %s: aspell %s",
                     ASPELL_PLUGIN_NAME, aspell_version_string ());
 #else
     weechat_printf (NULL, "  %s: aspell (?)", ASPELL_PLUGIN_NAME);
-#endif
+#endif /* HAVE_ASPELL_VERSION_STRING */
 #endif /* USE_ENCHANT */
 
     return WEECHAT_RC_OK;
