@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2010-2014 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -17,8 +17,8 @@
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __WEECHAT_IRC_NOTIFY_H
-#define __WEECHAT_IRC_NOTIFY_H 1
+#ifndef WEECHAT_IRC_NOTIFY_H
+#define WEECHAT_IRC_NOTIFY_H 1
 
 struct t_irc_server;
 
@@ -54,9 +54,12 @@ extern void irc_notify_check_now (struct t_irc_notify *notify);
 extern void irc_notify_new_for_server (struct t_irc_server *server);
 extern void irc_notify_new_for_all_servers ();
 extern void irc_notify_free (struct t_irc_server *server,
-                             struct t_irc_notify *notify);
+                             struct t_irc_notify *notify, int remove_monitor);
+extern void irc_notify_set_is_on_server (struct t_irc_notify *notify,
+                                         const char *host, int is_on_server);
 extern void irc_notify_free_all (struct t_irc_server *server);
 extern void irc_notify_display_list (struct t_irc_server *server);
+extern void irc_notify_send_monitor (struct t_irc_server *server);
 extern int irc_notify_timer_ison_cb (void *data, int remaining_calls);
 extern int irc_notify_timer_whois_cb (void *data, int remaining_calls);
 extern struct t_hdata *irc_notify_hdata_notify_cb (void *data,
@@ -69,4 +72,4 @@ extern void irc_notify_hook_timer_whois ();
 extern void irc_notify_init ();
 extern void irc_notify_end ();
 
-#endif /* __WEECHAT_IRC_NOTIFY_H */
+#endif /* WEECHAT_IRC_NOTIFY_H */
