@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -17,26 +17,27 @@
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __WEECHAT_LOGGER_CONFIG_H
-#define __WEECHAT_LOGGER_CONFIG_H 1
+#ifndef WEECHAT_LOGGER_CONFIG_H
+#define WEECHAT_LOGGER_CONFIG_H 1
 
 #define LOGGER_CONFIG_NAME "logger"
 
 
 extern struct t_config_option *logger_config_look_backlog;
 
-extern struct t_config_option *logger_config_color_backlog_line;
 extern struct t_config_option *logger_config_color_backlog_end;
+extern struct t_config_option *logger_config_color_backlog_line;
 
 extern struct t_config_option *logger_config_file_auto_log;
-extern struct t_config_option *logger_config_file_name_lower_case;
-extern struct t_config_option *logger_config_file_path;
-extern struct t_config_option *logger_config_file_mask;
-extern struct t_config_option *logger_config_file_replacement_char;
+extern struct t_config_option *logger_config_file_flush_delay;
 extern struct t_config_option *logger_config_file_info_lines;
-extern struct t_config_option *logger_config_file_time_format;
+extern struct t_config_option *logger_config_file_mask;
+extern struct t_config_option *logger_config_file_name_lower_case;
 extern struct t_config_option *logger_config_file_nick_prefix;
 extern struct t_config_option *logger_config_file_nick_suffix;
+extern struct t_config_option *logger_config_file_path;
+extern struct t_config_option *logger_config_file_replacement_char;
+extern struct t_config_option *logger_config_file_time_format;
 
 extern struct t_config_option *logger_config_get_level (const char *name);
 extern int logger_config_set_level (const char *name, const char *value);
@@ -46,4 +47,4 @@ extern int logger_config_read ();
 extern int logger_config_write ();
 extern void logger_config_free ();
 
-#endif /* __WEECHAT_LOGGER_CONFIG_H */
+#endif /* WEECHAT_LOGGER_CONFIG_H */
