@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006-2007 Emmanuel Bouthenot <kolter@openics.org>
- * Copyright (C) 2006-2013 Sebastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2006-2014 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -18,8 +18,8 @@
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __WEECHAT_LUA_H
-#define __WEECHAT_LUA_H 1
+#ifndef WEECHAT_LUA_H
+#define WEECHAT_LUA_H 1
 
 #define weechat_plugin weechat_lua_plugin
 #define LUA_PLUGIN_NAME "lua"
@@ -50,11 +50,12 @@ extern struct t_hashtable *weechat_lua_tohashtable (lua_State *interpreter,
                                                     int size,
                                                     const char *type_keys,
                                                     const char *type_values);
-extern void *weechat_lua_exec (struct t_plugin_script *script, int ret_type,
+extern void *weechat_lua_exec (struct t_plugin_script *script,
+                               int ret_type,
                                const char *function,
                                const char *format, void **argv);
 extern void weechat_lua_register_lib(lua_State *L, const char *libname,
                                      const luaL_Reg *lua_api_funcs,
                                      struct t_lua_const lua_api_consts[]);
 
-#endif /* __WEECHAT_LUA_H */
+#endif /* WEECHAT_LUA_H */
